@@ -10,6 +10,7 @@ from __future__ import with_statement
 import _ast
 import atexit
 import collections
+import csv
 import functools
 import optparse
 import os
@@ -25,14 +26,28 @@ try:                    # Python 3
     from xmlrpc.client import Fault, ServerProxy
     basestring = str
     int_types = int
+    _DictWriter = csv.DictWriter
 except ImportError:     # Python 2
     import ConfigParser as configparser
     from threading import currentThread as current_thread
     from xmlrpclib import Fault, ServerProxy
     int_types = int, long
 
+    class _DictWriter(csv.DictWriter):
+        """Unicode CSV Writer, which encodes output to UTF-8."""
 
-__version__ = '1.5.2'
+        def writeheader(self):
+            # Method 'writeheader' does not exist in Python 2.6
+            header = dict(zip(self.fieldnames, self.fieldnames))
+            self.writerow(header)
+
+        def _dict_to_list(self, rowdict):
+            rowlst = csv.DictWriter._dict_to_list(self, rowdict)
+            return [cell.encode('utf-8') if hasattr(cell, 'encode') else cell
+                    for cell in rowlst]
+
+
+__version__ = '1.5.3'
 __all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service',
            'format_exception', 'read_config', 'start_openerp_services']
 
@@ -411,7 +426,7 @@ class Client(object):
         """
         (server, db, user, password) = read_config(environment)
         if server[0] == 'local':
-            appname = os.path.basename(__file__)
+            appname = os.path.basename(__file__).rstrip('co')
             server = start_openerp_services(server[1], appname=appname)
         client = cls(server, db, user, password, verbose=verbose)
         client._environment = environment
@@ -705,7 +720,10 @@ class Client(object):
         mods = self.read('ir.module.module',
                          [('state', 'not in', STABLE_STATES)], 'name state')
         if not mods:
-            if modules:
+            if ids:
+                print('Already up-to-date: %s' %
+                      self.modules([('id', 'in', ids)]))
+            elif modules:
                 print('Module(s) not found: %s' % ', '.join(modules))
             else:
                 print('%s module(s) updated' % updated)
@@ -858,7 +876,10 @@ class Client(object):
         The return value is a dictionary where module names are grouped in
         lists according to their ``state``.
         """
-        domain = [('name', 'like', name)]
+        if isinstance(name, basestring):
+            domain = [('name', 'like', name)]
+        else:
+            domain = name
         if installed is not None:
             op = 'not in' if installed else 'in'
             domain.append(('state', op, ['uninstalled', 'uninstallable']))
@@ -867,7 +888,7 @@ class Client(object):
             res = collections.defaultdict(list)
             for mod in mods:
                 res[mod['state']].append(mod['name'])
-            return res
+            return dict(res)
 
     def keys(self, obj):
         """Wrapper for :meth:`Model.keys` method."""
@@ -1547,8 +1568,16 @@ def main():
                         verbose=args.verbose)
 
     if args.model and domain and client.user:
-        data = client.execute(args.model, 'read', domain, args.fields)
-        pprint(data)
+        context = {'lang': (os.environ.get('LANG') or 'en_US').split('.')[0]}
+        data = client.execute(args.model, 'read', domain, args.fields, context)
+        if not args.fields:
+            args.fields = ['id']
+            if data:
+                args.fields.extend([fld for fld in data[0] if fld != 'id'])
+        writer = _DictWriter(sys.stdout, args.fields, "", "ignore",
+                             quoting=csv.QUOTE_NONNUMERIC)
+        writer.writeheader()
+        writer.writerows(data)
 
     if client.connect is not None:
         # Set the globals()
